@@ -34,6 +34,7 @@ import json
 #Bio-specific packages
 import pyBigWig
 import pysam
+from pybedtools import BedTool
 #import MOODS.scan
 #import MOODS.tools
 #import MOODS.parsers
@@ -58,13 +59,25 @@ def sigmoid(x, a, b, L, shift):
 	y = L / (1 + np.exp(-b*(x-a))) + shift
 	return y
 
+
 class ArrayNorm:
 
-	def __init__(self, popt):
-		self.popt = popt
+	def __init__(self, p, value_max):
+
+		self.p_func = p	#function for getting normalization factor
+
+		self.value_max = value_max	#value max is set to prevent spurious normalization factors at high values,
+									#which can happen due to oscillations from p_func() at high number of degrees
 
 	def normalize(self, arr):
-		return(arr * sigmoid(arr, *self.popt))
+		""" Arr can be a value (float/int) or a numpy array """
+
+		#Set the cap for arr at self.value_max
+		#this prevents outliers of breaking the previously predicted p_func
+		arr_capped = arr * (arr <= self.value_max) + self.value_max * (arr > self.value_max)
+
+		return(arr * self.p_func(arr_capped))
+
 
 def dict_to_tab(dict_list, fname, chosen_columns, header=False):
 
@@ -86,48 +99,83 @@ def dict_to_tab(dict_list, fname, chosen_columns, header=False):
 	f.close()
 
 #Quantile normalization 
-def quantile_normalization(list_of_arrays): #lists paired values to normalize
+def quantile_normalization(list_of_arrays, names, pdfpages=None): #lists paired values to normalize
 
-	n = len(list_of_arrays) #number of arrays to normalize
+	n = len(list_of_arrays)
+	norm_objects = {}	#keys will be the strings in 'names'
 
 	#Calculate 
-	quantiles = np.linspace(0.2,0.999,500)
-	array_quantiles = [np.quantile(arr[arr > 0], quantiles) for arr in list_of_arrays]
+	quantiles = np.linspace(0.05,1,500, endpoint=True)	#the lower bound excludes most 0's from quantiles
+	array_quantiles = [np.quantile([0] if sum(arr > 0) == 0 else arr[arr > 0], quantiles) for arr in list_of_arrays]
 	mean_array_quantiles = [np.mean([array_quantiles[i][j] for i in range(n)]) for j in range(len(quantiles))]
 
-	norm_objects = []
-	for i in range(n):
+	#Plot overview of quantile-quantile
+	if pdfpages is not None:
+		fig, ax = plt.subplots()
+		for i in range(n):
+			plt.plot(array_quantiles[i], mean_array_quantiles, label="Quantiles for '{0}'".format(names[i]))
 		
-		#Plot q-q
-		#f, ax = plt.subplots()
-		#plt.scatter(array_quantiles[i], mean_array_quantiles)
-		#ax.set_xlim(0,1)
-		#ax.set_ylim(0,1)
-		#ax.plot([0, 1], [0, 1], transform=ax.transAxes)
-		#plt.close()
-		#plt.show()
-		
-		#Plot normalizyation factors
-		#f, ax = plt.subplots()
-		xdata = array_quantiles[i]
-		ydata = mean_array_quantiles/xdata
-		#plt.scatter(xdata, ydata)
-		#plt.close()
-
-		popt, pcov = curve_fit(sigmoid, xdata, ydata, bounds=((0,-np.inf, 0, 0), (np.inf, np.inf, np.inf, np.inf)))
-		norm_objects.append(ArrayNorm(popt))
-
-		y_est = sigmoid(xdata, *popt)
-		plt.plot(xdata, y_est)
-
+		plt.title("Quantile-quantile plot")
+		ax.set_xlabel("Value quantiles")
+		ax.set_ylabel("Mean quantiles")
+		ax.plot([0, 1], [0, 1], transform=ax.transAxes, linestyle="dashed", color="black", label="Expected")
+		plt.legend()
+		pdfpages.savefig(fig, bbox_inches='tight')
 		plt.close()
 
-	#Normalize each array using means per rank
-	normed = []
-	for i in range(n):	#for each array
-		normed.append(norm_objects[i].normalize(list_of_arrays[i]))
+	#Estimate necessary correction
+	for i, bigwig in enumerate(names):
 
-	return((normed, norm_objects))
+		xdata = array_quantiles[i]
+		ydata = mean_array_quantiles/xdata	#gives NA if xdata is 0
+
+		fig, ax = plt.subplots(nrows=2, ncols=1, constrained_layout=True)
+		residual_list = []
+		poly_functions = {}
+		degrees = list(range(1,10))
+		for deg in degrees:
+
+			o = np.polyfit(xdata, ydata, deg, full=True)
+			z, residuals, rank, singular_values, rcond = o	#unpack values
+
+			if len(residuals) == 0:
+				residuals = [0]	#if residuals is empty, the fit is exact
+
+			residual_list.append(residuals[0])
+			p = np.poly1d(z)
+			poly_functions[deg] = p
+			ax[0].plot(xdata, p(xdata), linestyle='dashed', label="fit deg = {0} (residual: {1})".format(deg, round(residuals[0], 3)))
+	
+		ax[0].set_xlabel("Original value")
+		ax[0].set_ylabel("Multiplication factor")
+		ax[0].set_title("Multiplication needed for normalization of '{0}'".format(bigwig))
+		ax[0].plot(xdata, ydata, color="black", linewidth=2, label="Original")
+		ax[0].legend(loc='center left', bbox_to_anchor=(1, 0.5))
+
+		#Use knee-plot to estimate how many degrees to use
+		try:
+			kneedle = KneeLocator(degrees, residual_list, S=1, curve='convex', direction='decreasing')
+			n_degrees = kneedle.knee + 1
+
+		except: #if knee was not found
+			n_degrees = 5
+
+		ax[1].set_title("Number of degrees chosen for fit")
+		ax[1].plot(degrees, residual_list)
+		ax[1].axvline(n_degrees, label="Number of degrees chosen", color="black")
+		ax[1].set_xlabel("Number of degrees")
+		ax[1].set_ylabel("Residuals of fit")
+		ax[1].legend(loc='center left', bbox_to_anchor=(1, 0.5))
+
+		#plt.adjust_subplots()
+
+		if pdfpages is not None:
+			pdfpages.savefig(fig,  bbox_inches="tight")
+		plt.close()
+
+		norm_objects[bigwig] = ArrayNorm(poly_functions[n_degrees], value_max=np.max(xdata))
+
+	return(norm_objects)
 
 def plot_score_distribution(list_of_arr, labels=[], title="Score distribution"):
 
@@ -189,7 +237,7 @@ def scan_and_score(regions, motifs_obj, args, log_q, qs):
 
 	######## Scan for motifs in each region ######
 	logger.debug("Scanning for motif occurrences")
-	all_TFBS = {TF: RegionList() for TF in motifs_obj.names} 	# Dict for saving sites before writing
+	all_TFBS = {motif.prefix: RegionList() for motif in motifs_obj} 	# Dict for saving sites before writing
 	for i, region in enumerate(regions):
 		logger.spam("Processing region: {0}".format(region.tup()))
 	
@@ -209,7 +257,7 @@ def scan_and_score(regions, motifs_obj, args, log_q, qs):
 		#Read footprints in region
 		footprints = {}
 		for condition in args.cond_names:
-			footprints[condition] = region.get_signal(pybw[condition], logger=logger)
+			footprints[condition] = region.get_signal(pybw[condition], logger=logger, key=condition)
 				
 			if len(footprints[condition]) == 0:
 				logger.error("ERROR IN REGION: {0}".format(region))
@@ -221,7 +269,7 @@ def scan_and_score(regions, motifs_obj, args, log_q, qs):
 
 		#Scan for motifs across sequence from fasta
 		seq = fasta_obj.fetch(region.chrom, region.start, region.end)
-		region_TFBS = motifs_obj.scan_sequence(seq, region)		#RegionList of TFBS
+		region_TFBS = motifs_obj.scan_sequence(seq, region)		#RegionList of TFBS, automatically scanned on both plus/minus strands
 
 		#Extend all TFBS with extra columns from peaks and bigwigs 
 		extra_columns = region
@@ -238,7 +286,7 @@ def scan_and_score(regions, motifs_obj, args, log_q, qs):
 
 		#Split regions to single TFs
 		for TFBS in region_TFBS:
-			all_TFBS[TFBS.name].append(TFBS)
+			all_TFBS[TFBS.name].append(TFBS)	#TFBS.name is the prefix of motif
 
 	####### All input regions have been scanned #######
 	global_TFBS = RegionList()	#across all TFs
@@ -278,9 +326,21 @@ def process_tfbs(TF_name, args, log2fc_params): 	#per tf
 	#Pre-scanned sites to read
 	bed_outdir = os.path.join(args.outdir, TF_name, "beds")
 	filename = os.path.join(bed_outdir, TF_name + ".tmp")
+	tmp_files = [filename]
 	no_cond = len(args.cond_names)
 	comparisons = args.comparisons
 
+	#Set distribution function 
+	diff_dist = scipy.stats.norm
+	
+	#Subset analysis to args.output_peaks if these were given
+	if args.output_peaks is not None:
+		output_peaks_bt = BedTool(args.output_peaks)
+		sites_bt = BedTool(filename)
+		intersection = sites_bt.intersect(output_peaks_bt, u=True)
+		filename = intersection.fn	#Overwrite filename with the path to the bedtools object
+		tmp_files.append(intersection.fn)
+
 	#Read file to list of dicts
 	stime = datetime.now()
 	header = ["TFBS_chr", "TFBS_start", "TFBS_end", "TFBS_name", "TFBS_score", "TFBS_strand"] + args.peak_header_list + ["{0}_score".format(condition) for condition in args.cond_names]
@@ -306,7 +366,15 @@ def process_tfbs(TF_name, args, log2fc_params): 	#per tf
 		for condition in args.cond_names:
 			threshold = args.thresholds[condition]
 			line[condition + "_score"] = float(line[condition + "_score"])
-			line[condition + "_score"] = round(args.norm_objects[condition].normalize(line[condition + "_score"] ), 5)
+			original = line[condition + "_score"]
+
+			line[condition + "_score"] = args.norm_objects[condition].normalize(line[condition + "_score"])  #normalize score
+			line[condition + "_score"] = line[condition + "_score"] if line[condition + "_score"]  > 0 else 0 # any scores below 0 -> 0
+			line[condition + "_score"] = round(line[condition + "_score"], 5)
+
+			if line[condition + "_score"] < 0:
+				logger.error("negative values: {0}. Original: {1}".format(line[condition + "_score"], original))
+
 			line[condition + "_bound"] = 1 if line[condition + "_score"] > threshold else 0
 
 		#Comparison specific
@@ -395,9 +463,9 @@ def process_tfbs(TF_name, args, log2fc_params): 	#per tf
 
 			#Estimate mean/std
 			bg_params = log2fc_params[(cond1, cond2)]
-			obs_params = scipy.stats.norm.fit(observed_log2fcs)
+			obs_params = diff_dist.fit(observed_log2fcs)
 
-			obs_mean, obs_std = obs_params
+			obs_mean, obs_std = obs_params	#only for scipy.stats.norm
 			bg_mean, bg_std = bg_params
 			obs_no = np.min([len(observed_log2fcs), 50000])		#Set cap on obs_no to prevent super small p-values
 			n_obs = len(observed_log2fcs)
@@ -407,10 +475,6 @@ def process_tfbs(TF_name, args, log2fc_params): 	#per tf
 				info_table.at[TF_name, base + "_change"] = (obs_mean - bg_mean) / np.mean([obs_std, bg_std])  #effect size
 				info_table.at[TF_name, base + "_change"] = np.round(info_table.at[TF_name, base + "_change"], 5)
 			
-				#pval = scipy.stats.mannwhitneyu(observed_log2fcs, bg_log2fcs, alternative="two-sided")[1]
-				#pval = scipy.stats.ttest_ind_from_stats(obs_mean, obs_std, obs_no, bg_mean, bg_std, obs_no, equal_var=False)[1] 	#pvalue is second in tup
-				#info_table.at[TF_name, base + "_pvalue"] = pval
-			
 			#Else not possible to compare groups
 			else:
 				info_table.at[TF_name, base + "_change"] = 0
@@ -420,7 +484,7 @@ def process_tfbs(TF_name, args, log2fc_params): 	#per tf
 			np.random.seed(n_obs)
 			sample_changes = []
 			for i in range(100):
-				sample = scipy.stats.norm.rvs(*log2fc_params[(cond1, cond2)], size=n_obs)	
+				sample = diff_dist.rvs(*log2fc_params[(cond1, cond2)], size=n_obs)	
 				sample_mean, sample_std = np.mean(sample), np.std(sample)
 				sample_change = (sample_mean - bg_mean) / np.mean([sample_std, bg_std])
 				sample_changes.append(sample_change)
@@ -434,19 +498,19 @@ def process_tfbs(TF_name, args, log2fc_params): 	#per tf
 			#Estimate p-value by comparing sampling to observed mean
 			ttest = scipy.stats.ttest_1samp(sample_changes, info_table.at[TF_name, base + "_change"])
 			info_table.at[TF_name, base + "_pvalue"] = ttest[1]
-			
+
 			#### Plot comparison ###
 			fig, ax = plt.subplots(1,1)
 			ax.hist(observed_log2fcs, bins='auto', label="Observed log2fcs", density=True)
 			xvals = np.linspace(plt.xlim()[0], plt.xlim()[1], 100)
 			
 			#Observed distribution
-			pdf = scipy.stats.norm.pdf(xvals, *obs_params)
+			pdf = diff_dist.pdf(xvals, *obs_params)
 			ax.plot(xvals, pdf, label="Observed distribution (fit)", color="red", linestyle="--")
 			ax.axvline(obs_mean, color="red", label="Observed mean")
 			
 			#Background distribution
-			pdf = scipy.stats.norm.pdf(xvals, *bg_params)
+			pdf = diff_dist.pdf(xvals, *bg_params)
 			ax.plot(xvals, pdf, label="Background distribution (fit)", color="Black", linestyle="--")
 			ax.axvline(bg_mean, color="black", label="Background mean")
 
@@ -475,10 +539,12 @@ def process_tfbs(TF_name, args, log2fc_params): 	#per tf
 	logger.spam("{0} - Global effects took:\t{1}".format(TF_name, etime - stime))
 
 	#################### Remove temporary file ######################
-	try:
-		os.remove(filename)
-	except:
-		logger.error("Could not remove temporary file {0} - this does not effect the results of BINDetect.".format(filename) )
+
+	for filename in tmp_files:
+		try:
+			os.remove(filename)
+		except:
+			logger.error("Could not remove temporary file {0} - this does not effect the results of BINDetect.".format(filename) )
 
 	return(info_table)
 
@@ -496,7 +562,7 @@ def plot_bindetect(motifs, cluster_obj, conditions, args):
 	warnings.filterwarnings("ignore")
 
 	cond1, cond2 = conditions
-	no_IDS = cluster_obj.n
+	n_IDS = cluster_obj.n
 
 	#Link information from motifs / clusters
 	diff_scores = {}
@@ -546,8 +612,8 @@ def plot_bindetect(motifs, cluster_obj, conditions, args):
 
 	#Make figure
 	no_rows, no_cols = 2,2	
-	h_ratios = [1,max(1,no_IDS/25)]
-	figsize = (8,10+7*(no_IDS/25))
+	h_ratios = [1,max(1,n_IDS/25)]
+	figsize = (8,10+7*(n_IDS/25))
 	
 	fig = plt.figure(figsize = figsize)
 	gs = gridspec.GridSpec(no_rows, no_cols, height_ratios=h_ratios)
@@ -581,17 +647,23 @@ def plot_bindetect(motifs, cluster_obj, conditions, args):
 
 	########### Dendrogram over similarities of TFs #######
 	
-	dendro_dat = dendrogram(cluster_obj.linkage_mat, labels=list(IDS), no_labels=True, orientation="right", ax=ax3, above_threshold_color="black", link_color_func=lambda k: cluster_obj.node_color[k])
-	labels = dendro_dat["ivl"]	#Now sorted for the order in dendrogram
-	ax3.set_xlabel("Transcription factor similarities\n(Clusters below threshold are colored)")
+	#Only plot dendrogram if there was more than one TF
+	n_ids = len(IDS)
+	if n_ids > 1:
+		dendro_dat = dendrogram(cluster_obj.linkage_mat, labels=list(IDS), no_labels=True, orientation="right", ax=ax3, above_threshold_color="black", link_color_func=lambda k: cluster_obj.node_color[k])
+		labels = dendro_dat["ivl"]	#Now sorted for the order in dendrogram
+		ax3.set_xlabel("Transcription factor similarities\n(Clusters below threshold are colored)")
 
-	ax3.set_ylabel("Transcription factor clustering based on TFBS overlap", rotation=270, labelpad=20)
-	ax3.yaxis.set_label_position("right")
+		ax3.set_ylabel("Transcription factor clustering based on TFBS overlap", rotation=270, labelpad=20)
+		ax3.yaxis.set_label_position("right")
 
-	#Set aspect of dendrogram/changes
-	x0,x1 = ax3.get_xlim()
-	y0,y1 = ax3.get_ylim()
-	ax3.set_aspect(((x1-x0)/(y1-y0)) * no_IDS/10)		
+		#Set aspect of dendrogram/changes
+		x0,x1 = ax3.get_xlim()
+		y0,y1 = ax3.get_ylim()
+		ax3.set_aspect(((x1-x0)/(y1-y0)) * n_ids/10)
+	else:
+		ax3.axis('off')
+		labels = IDS
 
 	########## Differential binding scores per TF ##########
 	ax2.set_xlabel("Differential binding score\n" + "(" + cond2 + r' $\leftarrow$' + r'$\rightarrow$ ' + cond1 + ")") #First position in comparison equals numerator in log2fc division
@@ -630,7 +702,7 @@ def plot_bindetect(motifs, cluster_obj, conditions, args):
 	#set aspect
 	x0,x1 = ax2.get_xlim()
 	y0,y1 = ax2.get_ylim()
-	ax2.set_aspect(((x1-x0)/(y1-y0)) * no_IDS/10)		#square volcano plot
+	ax2.set_aspect(((x1-x0)/(y1-y0)) * n_IDS/10)		#square volcano plot
 
 	plt.tight_layout()    #tight layout before setting ids in volcano plot
 
